Imports System.Data.Entity
Imports CustomerOrder
Imports CustomerOrder.Entities
Imports DataLayer.Mapping

Namespace DataLayer
  Public Class CustomerOrderContext
	  Inherits DbContext

	Public Property Addresses() As DbSet(Of Address)
	Public Property Customers() As DbSet(Of Customer)

	Public Property Products() As DbSet(Of Product)
	Public Property ProductCategories() As DbSet(Of Category)
	Public Property ProductDescriptions() As DbSet(Of ProductDescription)
	Public Property ProductModels() As DbSet(Of ProductModel)
	Public Property Orders() As DbSet(Of Order)
	Public Property LineItems() As DbSet(Of LineItem)
	Public Property CustomerLites() As DbSet(Of CustomerLite)

	Public Overrides Function SaveChanges() As Integer
      For Each ctEntry In ChangeTracker.Entries().Where(Function(e) TypeOf e.Entity Is DateClass AndAlso (e.State = EntityState.Added Or e.State = EntityState.Modified))
        Dim dateClass = CType(ctEntry.Entity, DateClass)
        If ctEntry.State = EntityState.Added Then
          dateClass.AddedDate = Date.Now
          dateClass.ModifiedDate = Date.Now

        End If
        If ctEntry.State = EntityState.Modified Then
          dateClass.ModifiedDate = Date.Now
        End If
      Next ctEntry
	  Return MyBase.SaveChanges()
	End Function

	Protected Overrides Sub OnModelCreating(ByVal modelBuilder As DbModelBuilder)
	  modelBuilder.Configurations.Add(New OrderShippingMap())
	  modelBuilder.Configurations.Add(New CustomerLiteMap())
	  modelBuilder.Configurations.Add(New CustomerMap())
	  modelBuilder.Configurations.Add(New OrderMap())
	End Sub
  End Class

  Public Class MyInitializer
	  Inherits DropCreateDatabaseIfModelChanges(Of CustomerOrderContext)

	Public Sub New()
	 'web.config db initialization requires a parameterless constructor.
	  Dim x = 3
	End Sub
	Protected Overrides Sub Seed(ByVal context As CustomerOrderContext)


	  Dim choc = New Product With {.Name = "Bittersweet Chocolate", .ProductCategory = New Category With {.Name = "Baking"}}
	  Dim eggs = New Product With {.Name = "Eggs", .ProductCategory = New Category With {.Name = "Dairy"}}
	  Dim raisins = New Product With {.Name = "Raisins", .ProductCategory = New Category With {.Name = "Fruit"}}


      Dim TempList As New List(Of Customer)
      TempList.Add(
    New Customer With {.FirstName = "Julie", .MiddleName = "H", .LastName = "Lerman", .CompanyName = "The Data Farm", .Title = "Head Geek", .ModifiedDate = Date.Now,
         .Orders = New List(Of Order) From {
    New Order With {.ModifiedDate = Date.Now, .SalesOrderNumber = "12345", .DueDate = Date.Today + New TimeSpan(30, 0, 0, 0), .OrderDate = Date.Today,
                    .LineItems = New List(Of LineItem) From {
    New LineItem With {.Product = choc, .OrderQty = 4, .UnitPrice = 3D},
    New LineItem With {.Product = eggs, .OrderQty = 1, .UnitPrice = 3.5D}}, .SubTotal = 16.5D},
    New Order With {.ModifiedDate = Date.Now, .SalesOrderNumber = "98765", .OrderDate = New Date(2011, 5, 24), .DueDate = New Date(2011, 5, 24) + New TimeSpan(30, 0, 0, 0), .SubTotal = 5.5D,
                    .LineItems = New List(Of LineItem) From {
    New LineItem With {.Product = choc, .OrderQty = 1, .UnitPrice = 3D},
    New LineItem With {.Product = raisins, .OrderQty = 1, .UnitPrice = 2.5D}}}}})
      TempList.ForEach(Function(c) context.Customers.Add(c))
	  MyBase.Seed(context)

	End Sub
  End Class
End Namespace

