﻿Imports System.ComponentModel
Imports System.Data.Services.Client
Imports System.Collections.ObjectModel
Imports WP7App.CustomerOrderService
Imports WP7App.ViewModels


Namespace WP7App
  Public Class DataAccess
	  Implements INotifyPropertyChanged
	Private privateLineItems As ObservableCollection(Of LineItemViewModel)
	Public Property LineItems() As ObservableCollection(Of LineItemViewModel)
		Get
			Return privateLineItems
		End Get
		Private Set(ByVal value As ObservableCollection(Of LineItemViewModel))
			privateLineItems = value
		End Set
	End Property
	Private privateOrders As ObservableCollection(Of OrderViewModel)
	Public Property Orders() As ObservableCollection(Of OrderViewModel)
		Get
			Return privateOrders
		End Get
		Private Set(ByVal value As ObservableCollection(Of OrderViewModel))
			privateOrders = value
		End Set
	End Property
	Private privateCustomer As CustomerViewModel
	Public Property Customer() As CustomerViewModel
		Get
			Return privateCustomer
		End Get
		Private Set(ByVal value As CustomerViewModel)
			privateCustomer = value
		End Set
	End Property
	Private _context As CustomerOrderContext
	Private _currentCustomerEntity As Customer




	Public Sub New()
	  _context = New CustomerOrderContext(New Uri("http://localhost:18049/CustomerDataService.svc/"))
	  LineItems = New ObservableCollection(Of LineItemViewModel)()
	  Orders = New ObservableCollection(Of OrderViewModel)()
	  Customer = New CustomerViewModel()
	End Sub

	Private privateCurrentOrderNumber As String
	Public Property CurrentOrderNumber() As String
		Get
			Return privateCurrentOrderNumber
		End Get
		Private Set(ByVal value As String)
			privateCurrentOrderNumber = value
		End Set
	End Property
	Private privateIsDataLoaded As Boolean
	Public Property IsDataLoaded() As Boolean
		Get
			Return privateIsDataLoaded
		End Get
		Private Set(ByVal value As Boolean)
			privateIsDataLoaded = value
		End Set
	End Property

	Public Sub LoadData()
	  GetCustomerAndOrders()
	End Sub

	Private Sub GetCustomerAndOrders()
      Dim qry = CType((
       From c In _context.Customers.Expand("Orders")
       Select c).Take(1), DataServiceQuery(Of Customer))
      Try
        qry.BeginExecute(AddressOf CustomersAndOrdersRetrieved, qry)
      Catch ex As DataServiceQueryException
        Throw New ApplicationException("An error occurred during query execution.", ex)
      End Try

	End Sub

	Private Sub CustomersAndOrdersRetrieved(ByVal result As IAsyncResult)
	  Try
		Dim query = TryCast(result.AsyncState, DataServiceQuery(Of Customer))

		_currentCustomerEntity = TryCast(query.EndExecute(result).FirstOrDefault(), Customer)


		Customer = New CustomerViewModel With {.CustomerId = _currentCustomerEntity.CustomerId, .Email = _currentCustomerEntity.EmailAddress, .FirstName = _currentCustomerEntity.FirstName, .LastName = _currentCustomerEntity.LastName, .Phone = _currentCustomerEntity.Phone}

		Dim orders = New List(Of OrderViewModel)()
		For Each order In _currentCustomerEntity.Orders
		  orders.Add(New OrderViewModel With {.OrderId = order.OrderId, .OrderDate = order.OrderDate, .OrderNumber = order.SalesOrderNumber, .SubTotal = order.SubTotal})
		Next order

		Me.Orders = New ObservableCollection(Of OrderViewModel)(orders)

		NotifyPropertyChanged("Customer")
	  Catch ex As Exception

		NotifyPropertyChanged("Error:" & ex.Message)
	  End Try


	  Me.IsDataLoaded = True
	End Sub

	Public Sub Save()

	  _currentCustomerEntity.EmailAddress = Customer.Email
	  _currentCustomerEntity.FirstName = Customer.FirstName
	  _currentCustomerEntity.LastName = Customer.LastName
	  _currentCustomerEntity.Phone = Customer.Phone

	  _context.UpdateObject(_currentCustomerEntity)
	  _context.BeginSaveChanges(AddressOf FinishSaveChanges, _context)

	End Sub

	Private Sub FinishSaveChanges(ByVal result As IAsyncResult)
	  Try

		_context = TryCast(result.AsyncState, CustomerOrderContext)
		_context.EndSaveChanges(result)
        NotifySaveStatusChanged("Saved")

	  Catch ex As Exception

        NotifySaveStatusChanged("Error:" & ex.Message)
	  End Try
	End Sub
  

    Public Event PropertyChanged As PropertyChangedEventHandler Implements INotifyPropertyChanged.PropertyChanged
    Private Sub NotifyPropertyChanged(ByVal propertyName As String)
      Dim handler As PropertyChangedEventHandler = PropertyChangedEvent
      If handler IsNot Nothing Then
        handler(Me, New PropertyChangedEventArgs(propertyName))
      End If
    End Sub
    Public Delegate Sub SaveStatusChangedEventHandler(ByVal sender As Object, ByVal e As PropertyChangedEventArgs)

    Public Event SaveStatusChanged As SaveStatusChangedEventHandler
    Private Sub NotifySaveStatusChanged(ByVal status As String)
      Dim handler As SaveStatusChangedEventHandler = SaveStatusChangedEvent
      If Nothing IsNot handler Then
        handler(Me, New PropertyChangedEventArgs(status))
      End If
    End Sub


#Region "LineItems"

    Public Sub LoadLineItems(ByVal orderId As Integer)

      Me.CurrentOrderNumber = Orders.Where(Function(o) o.OrderId = orderId).FirstOrDefault().OrderNumber
      Dim qry = CType(
             (From l In _context.LineItems.Expand("Product")
             Where l.OrderId = orderId
             Select l), DataServiceQuery(Of LineItem))
      Try
        qry.BeginExecute(AddressOf OnLineItemsLoaded, qry)
      Catch ex As DataServiceQueryException
        Throw New ApplicationException("An error occurred during query execution.", ex)
      End Try

    End Sub

    Private Sub OnLineItemsLoaded(ByVal result As IAsyncResult)
      Try
        Dim query = TryCast(result.AsyncState, DataServiceQuery(Of LineItem))

        Dim lineItems = query.EndExecute(result).ToList()
        'avoid threading issue by modifying this locally scoped list
        Dim lineItemVMs = New List(Of LineItemViewModel)()

        For Each item In lineItems
          lineItemVMs.Add(New LineItemViewModel With {.OrderId = item.OrderId, .LineItemId = item.LineItemId, .ProductName = item.Product.Name, .Quantity = item.OrderQty, .UnitPrice = item.UnitPrice})
        Next item

        Me.LineItems = New ObservableCollection(Of LineItemViewModel)(lineItemVMs)
        NotifyPropertyChanged("LineItem")
      Catch ex As Exception

        NotifyPropertyChanged("Error:" & ex.Message)
      End Try


      Me.IsDataLoaded = True
    End Sub



#End Region

  End Class
End Namespace